@@ -38,12 +38,13 @@ extern "C" {
 #include <poll.h>
 }
 
-#include <cassert>
-#include <iostream>
+#include <map>
 #include <sstream>
+#include <utility>
 
 #include "atf/formats.hpp"
 #include "atf/parser.hpp"
+#include "atf/sanity.hpp"
 #include "atf/text.hpp"
 
 namespace impl = atf::formats;
@@ -55,6 +56,177 @@ namespace impl = atf::formats;
             func; \
     } while (false);
 
+// ------------------------------------------------------------------------
+// The "format_error" class.
+// ------------------------------------------------------------------------
+
+impl::format_error::format_error(const std::string& w) :
+    std::runtime_error(w.c_str())
+{
+}
+
+// ------------------------------------------------------------------------
+// The "header_entry" class.
+// ------------------------------------------------------------------------
+
+typedef std::map< std::string, std::string > attrs_map;
+
+class header_entry {
+    std::string m_name;
+    std::string m_value;
+    attrs_map m_attrs;
+
+public:
+    header_entry(void);
+    header_entry(const std::string&, const std::string&,
+                 attrs_map = attrs_map());
+
+    const std::string& name(void) const;
+    const std::string& value(void) const;
+    const attrs_map& attrs(void) const;
+    bool has_attr(const std::string&) const;
+    const std::string& get_attr(const std::string&) const;
+};
+
+typedef std::map< std::string, header_entry > headers_map;
+
+header_entry::header_entry(void)
+{
+}
+
+header_entry::header_entry(const std::string& n,
+                                 const std::string& v,
+                                 attrs_map as) :
+    m_name(n),
+    m_value(v),
+    m_attrs(as)
+{
+}
+
+const std::string&
+header_entry::name(void)
+    const
+{
+    return m_name;
+}
+
+const std::string&
+header_entry::value(void)
+    const
+{
+    return m_value;
+}
+
+const attrs_map&
+header_entry::attrs(void)
+    const
+{
+    return m_attrs;
+}
+
+bool
+header_entry::has_attr(const std::string& n)
+    const
+{
+    return m_attrs.find(n) != m_attrs.end();
+}
+
+const std::string&
+header_entry::get_attr(const std::string& n)
+    const
+{
+    attrs_map::const_iterator iter = m_attrs.find(n);
+    PRE(iter != m_attrs.end());
+    return (*iter).second;
+}
+
+// ------------------------------------------------------------------------
+// The header tokenizer.
+// ------------------------------------------------------------------------
+
+namespace header {
+
+static const atf::parser::token_type& eof_type = 0;
+static const atf::parser::token_type& nl_type = 1;
+static const atf::parser::token_type& text_type = 2;
+static const atf::parser::token_type& colon_type = 3;
+static const atf::parser::token_type& semicolon_type = 4;
+static const atf::parser::token_type& dblquote_type = 5;
+static const atf::parser::token_type& equal_type = 6;
+
+class tokenizer : public atf::parser::tokenizer< std::istream > {
+public:
+    tokenizer(std::istream& is, size_t curline) :
+        atf::parser::tokenizer< std::istream >
+            (is, true, eof_type, nl_type, text_type, curline)
+    {
+        add_delim(';', semicolon_type);
+        add_delim(':', colon_type);
+        add_delim('=', equal_type);
+        add_quote('"', dblquote_type);
+    }
+};
+
+static
+atf::parser::parser< header::tokenizer >&
+read(atf::parser::parser< header::tokenizer >& p, header_entry& he)
+{
+    using namespace header;
+
+    atf::parser::token t = p.expect(text_type, nl_type, "a header name");
+    if (t.type() == nl_type) {
+        he = header_entry();
+        return p;
+    }
+    std::string hdr_name = t.text();
+
+    t = p.expect(colon_type, "`:'");
+
+    t = p.expect(text_type, "a textual value");
+    std::string hdr_value = t.text();
+
+    attrs_map attrs;
+
+    for (;;) {
+        t = p.expect(eof_type, semicolon_type, nl_type,
+                     "eof, `;' or new line");
+        if (t.type() == eof_type || t.type() == nl_type)
+            break;
+
+        t = p.expect(text_type, "an attribute name");
+        std::string attr_name = t.text();
+
+        t = p.expect(equal_type, "`='");
+
+        t = p.expect(text_type, "word or quoted string");
+        std::string attr_value = t.text();
+        attrs[attr_name] = attr_value;
+    }
+
+    he = header_entry(hdr_name, hdr_value, attrs);
+
+    return p;
+}
+
+static
+std::ostream&
+write(std::ostream& os, const header_entry& he)
+{
+    std::string line = he.name() + ": " + he.value();
+    attrs_map as = he.attrs();
+    for (attrs_map::const_iterator iter = as.begin();
+         iter != as.end(); iter++) {
+        PRE((*iter).second.find('\"') == std::string::npos);
+        line += "; " + (*iter).first + "=\"" + (*iter).second + "\"";
+    }
+
+    os << line << std::endl;
+
+    return os;
+}
+
+} // namespace header
+
 // ------------------------------------------------------------------------
 // Auxiliary functions.
 // ------------------------------------------------------------------------
@@ -70,6 +242,95 @@ string_to_size_t(const std::string& str)
     return s;
 }
 
+static
+std::pair< size_t, headers_map >
+read_headers(std::istream& is, size_t curline)
+{
+    using impl::format_error;
+
+    headers_map hm;
+
+    //
+    // Grammar
+    //
+    // header = entry+ nl
+    // entry = line nl
+    // line = text colon text
+    //        (semicolon (text equal (text | dblquote string dblquote)))*
+    // string = quoted_string
+    //
+
+    header::tokenizer tkz(is, curline);
+    atf::parser::parser< header::tokenizer > p(tkz);
+
+    bool first = true;
+    for (;;) {
+        try {
+            header_entry he;
+            if (!header::read(p, he).good() || he.name().empty())
+                break;
+
+            if (first && he.name() != "Content-Type")
+                throw format_error("Could not determine content type");
+            else
+                first = false;
+
+            hm[he.name()] = he;
+        } catch (const atf::parser::parse_error& pe) {
+            p.add_error(pe);
+            p.reset(header::nl_type);
+        }
+    }
+
+    if (!is.good())
+        throw format_error("Unexpected end of stream");
+
+    return std::pair< size_t, headers_map >(tkz.lineno(), hm);
+}
+
+static
+void
+write_headers(const headers_map& hm,
+              std::ostream& os)
+{
+    PRE(!hm.empty());
+    headers_map::const_iterator ct = hm.find("Content-Type");
+    PRE(ct != hm.end());
+    header::write(os, (*ct).second);
+    for (headers_map::const_iterator iter = hm.begin(); iter != hm.end();
+         iter++) {
+        if ((*iter).first != "Content-Type")
+            header::write(os, (*iter).second);
+    }
+    os << std::endl;
+}
+
+static
+void
+validate_content_type(const headers_map& hm,
+                      const std::string& fmt,
+                      int version)
+{
+    using impl::format_error;
+
+    headers_map::const_iterator iter = hm.find("Content-Type");
+    if (iter == hm.end())
+        throw format_error("Could not determine content type");
+
+    const header_entry& he = (*iter).second;
+    if (he.value() != fmt)
+        throw format_error("Mismatched content type: expected `" + fmt +
+                           "' but got `" + he.value() + "'");
+
+    if (!he.has_attr("version"))
+        throw format_error("Could not determine version");
+    const std::string& vstr = atf::text::to_string(version);
+    if (he.get_attr("version") != vstr)
+        throw format_error("Mismatched version: expected `" +
+                           vstr + "' but got `" +
+                           he.get_attr("version") + "'");
+}
+
 // ------------------------------------------------------------------------
 // The "atf_atffile" auxiliary parser.
 // ------------------------------------------------------------------------
@@ -88,12 +349,11 @@ static const atf::parser::token_type prop_type = 8;
 static const atf::parser::token_type tp_type = 9;
 static const atf::parser::token_type tp_glob_type = 10;
 
-class tokenizer : public atf::parser::tokenizer
-    < atf::serial::internalizer > {
+class tokenizer : public atf::parser::tokenizer< std::istream > {
 public:
-    tokenizer(atf::serial::internalizer& is) :
-        atf::parser::tokenizer< atf::serial::internalizer >
-            (is, true, eof_type, nl_type, text_type, is.lineno())
+    tokenizer(std::istream& is, size_t curline) :
+        atf::parser::tokenizer< std::istream >
+            (is, true, eof_type, nl_type, text_type, curline)
     {
         add_delim(':', colon_type);
         add_delim('=', equal_type);
@@ -121,12 +381,11 @@ static const atf::parser::token_type& dblquote_type = 3;
 static const atf::parser::token_type& equal_type = 4;
 static const atf::parser::token_type& hash_type = 5;
 
-class tokenizer : public atf::parser::tokenizer
-    < atf::serial::internalizer > {
+class tokenizer : public atf::parser::tokenizer< std::istream > {
 public:
-    tokenizer(atf::serial::internalizer& is) :
-        atf::parser::tokenizer< atf::serial::internalizer >
-            (is, true, eof_type, nl_type, text_type, is.lineno())
+    tokenizer(std::istream& is, size_t curline) :
+        atf::parser::tokenizer< std::istream >
+            (is, true, eof_type, nl_type, text_type, curline)
     {
         add_delim('=', equal_type);
         add_delim('#', hash_type);
@@ -154,12 +413,11 @@ static const atf::parser::token_type& passed_type = 8;
 static const atf::parser::token_type& failed_type = 9;
 static const atf::parser::token_type& skipped_type = 10;
 
-class tokenizer : public atf::parser::tokenizer
-    < atf::serial::internalizer > {
+class tokenizer : public atf::parser::tokenizer< std::istream > {
 public:
-    tokenizer(atf::serial::internalizer& is) :
-        atf::parser::tokenizer< atf::serial::internalizer >
-            (is, true, eof_type, nl_type, text_type, is.lineno())
+    tokenizer(std::istream& is, size_t curline) :
+        atf::parser::tokenizer< std::istream >
+            (is, true, eof_type, nl_type, text_type, curline)
     {
         add_delim(':', colon_type);
         add_delim(',', comma_type);
@@ -185,9 +443,9 @@ static const atf::parser::token_type& nl_type = 1;
 static const atf::parser::token_type& text_type = 2;
 static const atf::parser::token_type& colon_type = 3;
 static const atf::parser::token_type& comma_type = 4;
-static const atf::parser::token_type& tps_count = 5;
-static const atf::parser::token_type& tp_start = 6;
-static const atf::parser::token_type& tp_end = 7;
+static const atf::parser::token_type& tps_count_type = 5;
+static const atf::parser::token_type& tp_start_type = 6;
+static const atf::parser::token_type& tp_end_type = 7;
 static const atf::parser::token_type& tc_start_type = 8;
 static const atf::parser::token_type& tc_so_type = 9;
 static const atf::parser::token_type& tc_se_type = 10;
@@ -195,19 +453,19 @@ static const atf::parser::token_type& tc_end_type = 11;
 static const atf::parser::token_type& passed_type = 12;
 static const atf::parser::token_type& failed_type = 13;
 static const atf::parser::token_type& skipped_type = 14;
+static const atf::parser::token_type& info_type = 16;
 
-class tokenizer : public atf::parser::tokenizer
-    < atf::serial::internalizer > {
+class tokenizer : public atf::parser::tokenizer< std::istream > {
 public:
-    tokenizer(atf::serial::internalizer& is) :
-        atf::parser::tokenizer< atf::serial::internalizer >
-            (is, true, eof_type, nl_type, text_type, is.lineno())
+    tokenizer(std::istream& is, size_t curline) :
+        atf::parser::tokenizer< std::istream >
+            (is, true, eof_type, nl_type, text_type, curline)
     {
         add_delim(':', colon_type);
         add_delim(',', comma_type);
-        add_keyword("tps-count", tps_count);
-        add_keyword("tp-start", tp_start);
-        add_keyword("tp-end", tp_end);
+        add_keyword("tps-count", tps_count_type);
+        add_keyword("tp-start", tp_start_type);
+        add_keyword("tp-end", tp_end_type);
         add_keyword("tc-start", tc_start_type);
         add_keyword("tc-so", tc_so_type);
         add_keyword("tc-se", tc_se_type);
@@ -215,6 +473,7 @@ public:
         add_keyword("passed", passed_type);
         add_keyword("failed", failed_type);
         add_keyword("skipped", skipped_type);
+        add_keyword("info", info_type);
     }
 };
 
@@ -225,7 +484,7 @@ public:
 // ------------------------------------------------------------------------
 
 impl::atf_atffile_reader::atf_atffile_reader(std::istream& is) :
-    m_int(is, "application/X-atf-atffile", 1)
+    m_is(is)
 {
 }
 
@@ -261,7 +520,10 @@ impl::atf_atffile_reader::read(void)
     using atf::parser::parse_error;
     using namespace atf_atffile;
 
-    tokenizer tkz(m_int);
+    std::pair< size_t, headers_map > hml = read_headers(m_is, 1);
+    validate_content_type(hml.second, "application/X-atf-atffile", 1);
+
+    tokenizer tkz(m_is, hml.first);
     atf::parser::parser< tokenizer > p(tkz);
 
     for (;;) {
@@ -308,7 +570,7 @@ impl::atf_atffile_reader::read(void)
             } else if (t.type() == nl_type) {
                 continue;
             } else
-                assert(false);
+                UNREACHABLE;
 
             t = p.expect(nl_type, hash_type, eof_type,
                          "new line or comment");
@@ -331,7 +593,7 @@ impl::atf_atffile_reader::read(void)
 // ------------------------------------------------------------------------
 
 impl::atf_config_reader::atf_config_reader(std::istream& is) :
-    m_int(is, "application/X-atf-config", 1)
+    m_is(is)
 {
 }
 
@@ -356,7 +618,10 @@ impl::atf_config_reader::read(void)
     using atf::parser::parse_error;
     using namespace atf_config;
 
-    tokenizer tkz(m_int);
+    std::pair< size_t, headers_map > hml = read_headers(m_is, 1);
+    validate_content_type(hml.second, "application/X-atf-config", 1);
+
+    tokenizer tkz(m_is, hml.first);
     atf::parser::parser< tokenizer > p(tkz);
 
     for (;;) {
@@ -385,7 +650,7 @@ impl::atf_config_reader::read(void)
                 }
             } else if (t.type() == nl_type) {
             } else
-                assert(false);
+                UNREACHABLE;
         } catch (const parse_error& pe) {
             p.add_error(pe);
             p.reset(nl_type);
@@ -400,7 +665,7 @@ impl::atf_config_reader::read(void)
 // ------------------------------------------------------------------------
 
 impl::atf_tcs_reader::atf_tcs_reader(std::istream& is) :
-    m_int(is, "application/X-atf-tcs", 1)
+    m_is(is)
 {
 }
 
@@ -494,7 +759,10 @@ impl::atf_tcs_reader::read(atf::io::unbuffered_istream& out,
     using atf::parser::parse_error;
     using namespace atf_tcs;
 
-    tokenizer tkz(m_int);
+    std::pair< size_t, headers_map > hml = read_headers(m_is, 1);
+    validate_content_type(hml.second, "application/X-atf-tcs", 1);
+
+    tokenizer tkz(m_is, hml.first);
     atf::parser::parser< tokenizer > p(tkz);
 
     try {
@@ -508,7 +776,7 @@ impl::atf_tcs_reader::read(atf::io::unbuffered_istream& out,
         t = p.expect(nl_type, "new line");
 
         size_t i = 0;
-        while (m_int.good() && i < ntcs) {
+        while (m_is.good() && i < ntcs) {
             try {
                 t = p.expect(tc_start_type, "start of test case");
 
@@ -558,7 +826,7 @@ impl::atf_tcs_reader::read(atf::io::unbuffered_istream& out,
                                           "test case result");
                     CALLBACK(p, got_tc_end(tests::tcr::skipped(reason)));
                 } else
-                    assert(false);
+                    UNREACHABLE;
 
                 t = p.expect(nl_type, "new line");
                 i++;
@@ -580,34 +848,46 @@ impl::atf_tcs_reader::read(atf::io::unbuffered_istream& out,
 // The "atf_tcs_writer" class.
 // ------------------------------------------------------------------------
 
-impl::atf_tcs_writer::atf_tcs_writer(std::ostream& os, size_t ntcs) :
-    m_ext(os, "application/X-atf-tcs", 1),
+impl::atf_tcs_writer::atf_tcs_writer(std::ostream& os,
+                                     std::ostream& p_cout,
+                                     std::ostream& p_cerr,
+                                     size_t ntcs) :
+    m_os(os),
+    m_cout(p_cout),
+    m_cerr(p_cerr),
     m_ntcs(ntcs),
     m_curtc(0)
 {
-    m_ext.putline("tcs-count: " + text::to_string(ntcs));
-    m_ext.flush();
+    headers_map hm;
+    attrs_map ct_attrs;
+    ct_attrs["version"] = "1";
+    hm["Content-Type"] =
+        header_entry("Content-Type", "application/X-atf-tcs", ct_attrs);
+    write_headers(hm, m_os);
+
+    m_os << "tcs-count: " << ntcs << std::endl;
+    m_os.flush();
 }
 
 void
 impl::atf_tcs_writer::start_tc(const std::string& tcname)
 {
     m_tcname = tcname;
-    m_ext.putline("tc-start: " + tcname);
-    m_ext.flush();
+    m_os << "tc-start: " << tcname << std::endl;
+    m_os.flush();
 }
 
 void
 impl::atf_tcs_writer::end_tc(const atf::tests::tcr& tcr)
 {
-    assert(m_curtc < m_ntcs);
+    PRE(m_curtc < m_ntcs);
     m_curtc++;
     if (m_curtc < m_ntcs) {
-        std::cout << "__atf_tc_separator__\n";
-        std::cerr << "__atf_tc_separator__\n";
+        m_cout << "__atf_tc_separator__\n";
+        m_cerr << "__atf_tc_separator__\n";
     }
-    std::cout.flush();
-    std::cerr.flush();
+    m_cout.flush();
+    m_cerr.flush();
 
     std::string end = "tc-end: " + m_tcname + ", ";
     switch (tcr.get_status()) {
@@ -624,10 +904,10 @@ impl::atf_tcs_writer::end_tc(const atf::tests::tcr& tcr)
         break;
 
     default:
-        assert(false);
+        UNREACHABLE;
     }
-    m_ext.putline(end);
-    m_ext.flush();
+    m_os << end << std::endl;
+    m_os.flush();
 }
 
 // ------------------------------------------------------------------------
@@ -635,7 +915,7 @@ impl::atf_tcs_writer::end_tc(const atf::tests::tcr& tcr)
 // ------------------------------------------------------------------------
 
 impl::atf_tps_reader::atf_tps_reader(std::istream& is) :
-    m_int(is, "application/X-atf-tps", 1)
+    m_is(is)
 {
 }
 
@@ -643,6 +923,12 @@ impl::atf_tps_reader::~atf_tps_reader(void)
 {
 }
 
+void
+impl::atf_tps_reader::got_info(const std::string& what,
+                               const std::string& val)
+{
+}
+
 void
 impl::atf_tps_reader::got_ntps(size_t ntps)
 {
@@ -683,6 +969,25 @@ impl::atf_tps_reader::got_eof(void)
 {
 }
 
+void
+impl::atf_tps_reader::read_info(void* pptr)
+{
+    using atf::parser::parse_error;
+    using namespace atf_tps;
+
+    atf::parser::parser< tokenizer >& p =
+        *reinterpret_cast< atf::parser::parser< tokenizer >* >
+        (pptr);
+
+    (void)p.expect(colon_type, "`:'");
+
+    atf::parser::token t = p.expect(text_type, "info property name");
+    (void)p.expect(comma_type, "`,'");
+    got_info(t.text(), atf::text::trim(p.rest_of_line()));
+
+    (void)p.expect(nl_type, "new line");
+}
+
 void
 impl::atf_tps_reader::read_tp(void* pptr)
 {
@@ -693,7 +998,7 @@ impl::atf_tps_reader::read_tp(void* pptr)
         *reinterpret_cast< atf::parser::parser< tokenizer >* >
         (pptr);
 
-    atf::parser::token t = p.expect(tp_start,
+    atf::parser::token t = p.expect(tp_start_type,
                                     "start of test program");
 
     t = p.expect(colon_type, "`:'");
@@ -720,7 +1025,7 @@ impl::atf_tps_reader::read_tp(void* pptr)
             p.reset(nl_type);
         }
     }
-    t = p.expect(tp_end, "end of test program");
+    t = p.expect(tp_end_type, "end of test program");
 
     t = p.expect(colon_type, "`:'");
 
@@ -772,12 +1077,12 @@ impl::atf_tps_reader::read_tc(void* pptr)
 
         t = p.expect(colon_type, "`:'");
 
-        std::string line = text::trim(p.rest_of_line());
+        std::string line = p.rest_of_line();
 
         if (t2.type() == tc_so_type) {
             CALLBACK(p, got_tc_stdout_line(line));
         } else {
-            assert(t2.type() == tc_se_type);
+            INV(t2.type() == tc_se_type);
             CALLBACK(p, got_tc_stderr_line(line));
         }
 
@@ -816,7 +1121,7 @@ impl::atf_tps_reader::read_tc(void* pptr)
                               "Empty reason for skipped test case result");
         CALLBACK(p, got_tc_end(tests::tcr::skipped(reason)));
     } else
-        assert(false);
+        UNREACHABLE;
 
     t = p.expect(nl_type, "new line");
 }
@@ -827,11 +1132,18 @@ impl::atf_tps_reader::read(void)
     using atf::parser::parse_error;
     using namespace atf_tps;
 
-    tokenizer tkz(m_int);
+    std::pair< size_t, headers_map > hml = read_headers(m_is, 1);
+    validate_content_type(hml.second, "application/X-atf-tps", 2);
+
+    tokenizer tkz(m_is, hml.first);
     atf::parser::parser< tokenizer > p(tkz);
 
     try {
-        atf::parser::token t = p.expect(tps_count, "tps-count field");
+        atf::parser::token t;
+
+        while ((t = p.expect(tps_count_type, info_type, "tps-count or info "
+                             "field")).type() == info_type)
+            read_info(&p);
 
         t = p.expect(colon_type, "`:'");
 
@@ -852,7 +1164,9 @@ impl::atf_tps_reader::read(void)
             }
         }
 
-        t = p.expect(eof_type, "end of stream");
+        while ((t = p.expect(eof_type, info_type, "end of stream or info "
+                             "field")).type() == info_type)
+            read_info(&p);
         CALLBACK(p, got_eof());
     } catch (const parse_error& pe) {
         p.add_error(pe);
@@ -864,51 +1178,70 @@ impl::atf_tps_reader::read(void)
 // The "atf_tps_writer" class.
 // ------------------------------------------------------------------------
 
-impl::atf_tps_writer::atf_tps_writer(std::ostream& os, size_t ntps) :
-    m_ext(os, "application/X-atf-tps", 1)
+impl::atf_tps_writer::atf_tps_writer(std::ostream& os) :
+    m_os(os)
 {
-    m_ext.putline("tps-count: " + text::to_string(ntps));
-    m_ext.flush();
+    headers_map hm;
+    attrs_map ct_attrs;
+    ct_attrs["version"] = "2";
+    hm["Content-Type"] =
+        header_entry("Content-Type", "application/X-atf-tps", ct_attrs);
+    write_headers(hm, m_os);
+}
+
+void
+impl::atf_tps_writer::info(const std::string& what, const std::string& val)
+{
+    m_os << "info: " << what << ", " << val << std::endl;
+    m_os.flush();
+}
+
+void
+impl::atf_tps_writer::ntps(size_t p_ntps)
+{
+    m_os << "tps-count: " << p_ntps << std::endl;
+    m_os.flush();
 }
 
 void
 impl::atf_tps_writer::start_tp(const std::string& tp, size_t ntcs)
 {
     m_tpname = tp;
-    m_ext.putline("tp-start: " + tp + ", " + text::to_string(ntcs));
-    m_ext.flush();
+    m_os << "tp-start: " << tp << ", " << ntcs << std::endl;
+    m_os.flush();
 }
 
 void
 impl::atf_tps_writer::end_tp(const std::string& reason)
 {
+    PRE(reason.find('\n') == std::string::npos);
     if (reason.empty())
-        m_ext.putline("tp-end: " + m_tpname);
+        m_os << "tp-end: " << m_tpname << std::endl;
     else
-        m_ext.putline("tp-end: " + m_tpname + ", " + reason);
-    m_ext.flush();
+        m_os << "tp-end: " << m_tpname << ", " << reason << std::endl;
+    m_os.flush();
 }
 
 void
 impl::atf_tps_writer::start_tc(const std::string& tcname)
 {
     m_tcname = tcname;
-    m_ext.putline("tc-start: " + tcname);
-    m_ext.flush();
+    m_os << "tc-start: " << tcname << std::endl;
+    m_os.flush();
 }
 
 void
 impl::atf_tps_writer::stdout_tc(const std::string& line)
 {
-    m_ext.putline("tc-so: " + line);
-    m_ext.flush();
+    m_os << "tc-so:" << line << std::endl;
+    m_os.flush();
 }
 
 void
 impl::atf_tps_writer::stderr_tc(const std::string& line)
 {
-    m_ext.putline("tc-se: " + line);
-    m_ext.flush();
+    m_os << "tc-se:" << line << std::endl;
+    m_os.flush();
 }
 
 void
@@ -929,8 +1262,8 @@ impl::atf_tps_writer::end_tc(const atf::tests::tcr& tcr)
         break;
 
     default:
-        assert(false);
+        UNREACHABLE;
     }
-    m_ext.putline(str);
-    m_ext.flush();
+    m_os << str << std::endl;
+    m_os.flush();
 }
