@@ -1,7 +1,7 @@
 //
 // Automated Testing Framework (atf)
 //
-// Copyright (c) 2007 The NetBSD Foundation, Inc.
+// Copyright (c) 2007, 2008 The NetBSD Foundation, Inc.
 // All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
@@ -198,7 +198,7 @@ merge_maps(std::map< K, V >& dest, const std::map< K, V >& src)
         dest[(*iter).first] = (*iter).second;
 }
 
-class atf_run : public atf::application {
+class atf_run : public atf::application::app {
     static const char* m_description;
 
     atf::tests::vars_map m_atffile_vars;
@@ -246,7 +246,7 @@ const char* atf_run::m_description =
     "results.";
 
 atf_run::atf_run(void) :
-    application(m_description, "atf-run(1)")
+    app(m_description, "atf-run(1)", "atf(7)")
 {
 }
 
@@ -278,6 +278,7 @@ atf_run::options_set
 atf_run::specific_options(void)
     const
 {
+    using atf::application::option;
     options_set opts;
     opts.insert(option('v', "var=value", "Sets the configuration variable "
                                          "`var' to `value'; overrides "
@@ -365,12 +366,12 @@ atf_run::run_test_program_child(const atf::fs::path& tp,
     // do not care to release it.  We are going to die anyway very soon,
     // either due to exec(2) or to exit(3).
     std::vector< std::string > confargs = conf_args();
-    char* args[4 + confargs.size()];
+    char** args = new char*[4 + confargs.size()];
     {
         // 0: Program name.
-        const char* name = tp.leaf_name().c_str();
-        args[0] = new char[std::strlen(name) + 1];
-        std::strcpy(args[0], name);
+        std::string progname = tp.leaf_name();
+        args[0] = new char[progname.length() + 1];
+        std::strcpy(args[0], progname.c_str());
 
         // 1: The file descriptor to which the results will be printed.
         args[1] = new char[4];
@@ -455,7 +456,7 @@ atf_run::run_test_program_parent(const atf::fs::path& tp,
     }
 
     int code, status;
-    if (::waitpid(pid, &status, 0) == -1) {
+    if (::waitpid(pid, &status, 0) != pid) {
         m.finalize("waitpid(2) on the child process " +
                    atf::text::to_string(pid) + " failed" +
                    (fmterr.empty() ? "" : (".  " + fmterr)));
@@ -478,11 +479,6 @@ atf_run::run_test_program_parent(const atf::fs::path& tp,
                        atf::text::to_string(WTERMSIG(status)) +
                        (WCOREDUMP(status) ? " (core dumped)" : "") +
                        (fmterr.empty() ? "" : (".  " + fmterr)));
-        } else if (WIFSTOPPED(status)) {
-            code = EXIT_FAILURE;
-            m.finalize("Test program stopped due to signal " +
-                       atf::text::to_string(WSTOPSIG(status)) +
-                       (fmterr.empty() ? "" : (".  " + fmterr)));
         } else
             throw std::runtime_error
                 ("Child process " + atf::text::to_string(pid) +
